from __future__ import print_function

import gc
import pickle
import logging
import threading
import os
import operator

from itertools import count
from math import sqrt
from random import random, choice

from scipy import where, array, exp, zeros, size, mat, median
from functools import reduce

# file extension for load/save protocol mapping
known_extensions = {
	'mat': 'matlab',
	'txt': 'ascii',
	'svm': 'libsvm',
	'pkl': 'pickle',
	'nc' : 'netcdf' }


def abstractMethod():
	""" This should be called when an abstract method is called that should have been
	implemented by a subclass. It should not be called in situations where no implementation
	(i.e. a 'pass' behavior) is acceptable. """
	raise NotImplementedError('Method not implemented!')


def drawIndex(probs, tolerant=False):
	""" Draws an index given an array of probabilities.

	:key tolerant: if set to True, the array is normalized to sum to 1.  """
	if not sum(probs) < 1.00001 or not sum(probs) > 0.99999:
		if tolerant:
			probs /= sum(probs)
		else:
			print((probs, 1 - sum(probs)))
			raise ValueError()
	r = random()
	s = 0
	for i, p in enumerate(probs):
		s += p
		if s > r:
			return i
	return choice(list(range(len(probs))))


def drawGibbs(vals, temperature=1.):
	""" Return the index of the sample drawn by a softmax (Gibbs). """
	if temperature == 0:
		# randomly pick one of the values with the max value.
		m = max(vals)
		best = []
		for i, v in enumerate(vals):
			if v == m:
				best.append(i)
		return choice(best)
	else:
		temp = vals / temperature

		# make sure we keep the exponential bounded (between +20 and -20)
		temp += 20 - max(temp)
		if min(temp) < -20:
			for i, v in enumerate(temp):
				if v < -20:
					temp[i] = -20
		temp = exp(temp)
		temp /= sum(temp)
		return drawIndex(temp)


def iterCombinations(tup):
	""" all possible of integer tuples of the same dimension than tup, and each component being
	positive and strictly inferior to the corresponding entry in tup. """
	if len(tup) == 1:
		for i in range(tup[0]):
			yield (i,)
	elif len(tup) > 1:
		for prefix in iterCombinations(tup[:-1]):
			for i in range(tup[-1]):
				yield tuple(list(prefix) + [i])


def setAllArgs(obj, argdict):
	""" set all those internal variables which have the same name than an entry in the
	given object's dictionary.
	This function can be useful for quick initializations. """

	xmlstore = isinstance(obj, XMLBuildable)
	for n in list(argdict.keys()):
		if hasattr(obj, n):
			setattr(obj, n, argdict[n])
			if xmlstore:
				obj.argdict[n] = argdict[n]
		else:
			print(('Warning: parameter name', n, 'not found!'))
			if xmlstore:
				if not hasattr(obj, '_unknown_argdict'):
					obj._unknown_argdict = {}
				obj._unknown_argdict[n] = argdict[n]


def linscale(d, lim):
	""" utility function to linearly scale array d to the interval defined by lim """
	return (d - d.min())*(lim[1] - lim[0]) + lim[0]


def percentError(out, true):
	""" return percentage of mismatch between out and target values (lists and arrays accepted) """
	arrout = array(out).flatten()
	wrong = where(arrout != array(true).flatten())[0].size
	return 100. * float(wrong) / float(arrout.size)


def formatFromExtension(fname):
	"""Tries to infer a protocol from the file extension."""
	_base, ext = os.path.splitext(fname)
	if not ext:
		return None
	try:
		format = known_extensions[ext.replace('.', '')]
	except KeyError:
		format = None
	return format


class XMLBuildable(object):
	""" subclasses of this can be losslessly stored in XML, and
	automatically reconstructed on reading. For this they need to store
	their construction arguments in the variable <argdict>. """

	argdict = None

	def setArgs(self, **argdict):
		if not self.argdict:
			self.argdict = {}
		setAllArgs(self, argdict)


class Serializable(object):
	"""Class that implements shortcuts to serialize an object.

	Serialization is done by various formats. At the moment, only 'pickle' is
	supported.
	"""

	def saveToFileLike(self, flo, format=None, **kwargs):
		"""Save the object to a given file like object in the given format.
		"""
		format = 'pickle' if format is None else format
		save = getattr(self, "save_%s" % format, None)
		if save is None:
			raise ValueError("Unknown format '%s'." % format)
		save(flo, **kwargs)

	@classmethod
	def loadFromFileLike(cls, flo, format=None):
		"""Load the object to a given file like object with the given protocol.
		"""
		format = 'pickle' if format is None else format
		load = getattr(cls, "load_%s" % format, None)
		if load is None:
			raise ValueError("Unknown format '%s'." % format)
		return load(flo)

	def saveToFile(self, filename, format=None, **kwargs):
		"""Save the object to file given by filename."""
		if format is None:
			# try to derive protocol from file extension
			format = formatFromExtension(filename)
		with open(filename, 'wb') as fp:
			self.saveToFileLike(fp, format, **kwargs)

	@classmethod
	def loadFromFile(cls, filename, format=None):
		"""Return an instance of the class that is saved in the file with the
		given filename in the specified format."""
		if format is None:
			# try to derive protocol from file extension
			format = formatFromExtension(filename)
		with open(filename, 'rbU') as fp:
			obj = cls.loadFromFileLike(fp, format)
			obj.filename = filename
			return obj

	def save_pickle(self, flo, protocol=0):
		pickle.dump(self, flo, protocol)

	@classmethod
	def load_pickle(cls, flo):
		return pickle.load(flo)


class Named(XMLBuildable):
	"""Class whose objects are guaranteed to have a unique name."""

	_nameIds = count(0)

	def getName(self):
		logging.warning("Deprecated, use .name property instead.")
		return self.name

	def setName(self, newname):
		logging.warning("Deprecated, use .name property instead.")
		self.name = newname

	def _getName(self):
		"""Returns the name, which is generated if it has not been already."""
		if self._name is None:
			self._name = self._generateName()
		return self._name

	def _setName(self, newname):
		"""Change name to newname. Uniqueness is not guaranteed anymore."""
		self._name = newname

	_name = None
	name = property(_getName, _setName)

	def _generateName(self):
		"""Return a unique name for this object."""
		return "%s-%i" % (self.__class__.__name__, next(self._nameIds))

	def __repr__(self):
		""" The default representation of a named object is its name. """
		return "<%s '%s'>" % (self.__class__.__name__, self.name)


def fListToString(a_list, a_precision=3):
	""" Returns a string representing a list of floats with a given precision """
	from numpy import around
	s_list = ", ".join(("%g" % around(x, a_precision)).ljust(a_precision+3)
					  for x in a_list)
	return "[%s]" % s_list

def tupleRemoveItem(tup, index):
	""" remove the item at position index of the tuple and return a new tuple. """
	l = list(tup)
	return tuple(l[:index] + l[index + 1:])


def confidenceIntervalSize(stdev, nbsamples):
	""" Determine the size of the confidence interval, given the standard deviation and the number of samples.
	t-test-percentile: 97.5%, infinitely many degrees of freedom,
	therefore on the two-sided interval: 95% """
	# CHECKME: for better precision, maybe get the percentile dynamically, from the scipy library?
	return 2 * 1.98 * stdev / sqrt(nbsamples)


def trace(func):
	def inner(*args, **kwargs):
		print(("%s: %s, %s" % (func.__name__, args, kwargs)))
		return func(*args, **kwargs)
	return inner


def threaded(callback=lambda * args, **kwargs: None, daemonic=False):
	"""Decorate  a function to run in its own thread and report the result
	by calling callback with it."""
	def innerDecorator(func):
		def inner(*args, **kwargs):
			target = lambda: callback(func(*args, **kwargs))
			t = threading.Thread(target=target)
			t.setDaemon(daemonic)
			t.start()
		return inner
	return innerDecorator


def garbagecollect(func):
	"""Decorate a function to invoke the garbage collector after each execution.
	"""
	def inner(*args, **kwargs):
		result = func(*args, **kwargs)
		gc.collect()
		return result
	return inner


def memoize(func):
	"""Decorate a function to 'memoize' results by holding it in a cache that
	maps call arguments to returns."""
	cache = {}
	def inner(*args, **kwargs):
		# Dictionaries and lists are unhashable
		args = tuple(args)
		# Make a set for checking in the cache, since the order of
		# .iteritems() is undefined
		kwargs_set = frozenset(iter(kwargs.items()))
		if (args, kwargs_set) in cache:
			result = cache[args, kwargs_set]
		else:
			result = func(*args, **kwargs)
			cache[args, kwargs_set] = result
		return result
	return inner


def storeCallResults(obj, verbose=False):
	"""Pseudo-decorate an object to store all evaluations of the function in the returned list."""
	results = []
	oldcall = obj.__class__.__call__
	def newcall(*args, **kwargs):
		result = oldcall(*args, **kwargs)
		results.append(result)
		if verbose:
			print(result)
		return result
	obj.__class__.__call__ = newcall
	return results


def multiEvaluate(repeat):
	"""Decorate a function to evaluate repeatedly with the same arguments, and return the average result """
	def decorator(func):
		def inner(*args, **kwargs):
			result = 0.
			for dummy in range(repeat):
				result += func(*args, **kwargs)
			return result / repeat
		return inner
	return decorator


def _import(name):
	"""Return module from a package.

	These two are equivalent:

		> from package import module as bar
		> bar = _import('package.module')

	"""
	mod = __import__(name)
	components = name.split('.')
	for comp in components[1:]:
		try:
			mod = getattr(mod, comp)
		except AttributeError:
			raise ImportError("No module named %s" % mod)
	return mod


# tools for binary Gray code manipulation:

def int2gray(i):
	""" Returns the value of an integer in Gray encoding."""
	return i ^ (i >> 1)


def gray2int(g, size):
	""" Transforms a Gray code back into an integer. """
	res = 0
	for i in reversed(list(range(size))):
		gi = (g >> i) % 2
		if i == size - 1:
			bi = gi
		else:
			bi = bi ^ gi
		res += bi * 2 ** i
	return res


def asBinary(i):
	""" Produces a string from an integer's binary representation.
	(preceding zeros removed). """
	if i > 1:
		if i % 2 == 1:
			return asBinary(i >> 1) + '1'
		else:
			return asBinary(i >> 1) + '0'
	else:
		return str(i)


def one_to_n(val, maxval):
	""" Returns a 1-in-n binary encoding of a non-negative integer. """
	a = zeros(maxval, float)
	a[val] = 1.
	return a


def n_to_one(arr):
	""" Returns the reverse of a 1-in-n binary encoding. """
	return where(arr == 1)[0][0]


def canonicClassString(x):
	""" the __class__ attribute changed from old-style to new-style classes... """
	if isinstance(x, object):
		return repr(x.__class__).split("'")[1]
	else:
		return repr(x.__class__)


def decrementAny(tup):
	""" the closest tuples to tup: decrementing by 1 along any dimension.
	Never go into negatives though. """
	res = []
	for i, x in enumerate(tup):
		if x > 0:
			res.append(tuple(list(tup[:i]) + [x - 1] + list(tup[i + 1:])))
	return res


def reachable(stepFunction, start, destinations, _alreadyseen=None):
	""" Determines the subset of destinations that can be reached from a set of starting positions,
	while using stepFunction (which produces a list of neighbor states) to navigate.
	Uses breadth-first search.
	Returns a dictionary with reachable destinations and their distances.
	"""
	if len(start) == 0 or len(destinations) == 0:
		return {}
	if _alreadyseen is None:
		_alreadyseen = []
	_alreadyseen.extend(start)

	# dict with distances to destinations
	res = {}
	for s in start:
		if s in destinations:
			res[s] = 0
			start.remove(s)

	# do one step
	new = set()
	for s in start:
		new.update(stepFunction(s))
	new.difference_update(_alreadyseen)
	ndestinations = list(destinations)

	for s in list(new):
		if s in destinations:
			res[s] = 1
			new.remove(s)
			ndestinations.remove(s)
			_alreadyseen.append(s)

	# recursively do the rest
	deeper = reachable(stepFunction, new, ndestinations, _alreadyseen)

	# adjust distances
	for k, val in list(deeper.items()):
		res[k] = val + 1
	return res


def flood(stepFunction, fullSet, initSet, relevant=None):
	""" Returns a list of elements of fullSet linked to some element of initSet
	through the neighborhood-setFunction (which must be defined on all elements of fullSet).

	:key relevant: (optional) list of relevant elements: stop once all relevant elements are found.
	"""
	if fullSet is None:
		flooded = set(initSet)
	else:
		full = set(fullSet)
		flooded = full.intersection(set(initSet))
		if relevant is None:
			relevant = full.copy()
	if relevant:
		relevant = set(relevant)

	change = flooded.copy()
	while len(change)>0:
		new = set()
		for m in change:
			if fullSet is None:
				new.update(stepFunction(m))
			else:
				new.update(full.intersection(stepFunction(m)))
		change = new.difference(flooded)
		flooded.update(change)
		if relevant is not None and relevant.issubset(flooded):
			break
	return list(flooded)


def crossproduct(ss, row=None, level=0):
	"""Returns the cross-product of the sets given in `ss`."""
	if row is None:
		row = []
	if len(ss) > 1:
		return reduce(operator.add,
					  [crossproduct(ss[1:], row + [i], level + 1) for i in ss[0]])
	else:
		return [row + [i] for i in ss[0]]


def permute(arr, permutation):
	"""Return an array like arr but with elements permuted.

	Only the first dimension is permuted, which makes it possible to permute
	blocks of the input.

	arr can be anything as long as it's indexable."""
	return array([arr[i] for i in permutation])


def permuteToBlocks(arr, blockshape):
	"""Permute an array so that it consists of linearized blocks.

	Example: A two-dimensional array of the form

		0  1  2  3
		4  5  6  7
		8  9  10 11
		12 13 14 15

	would be turned into an array like this with (2, 2) blocks:

		0 1 4 5 2 3 6 7 8 9 12 13 10 11 14 15
	"""
	if len(blockshape) < 2:
		raise ValueError("Need more than one dimension.")
	elif len(blockshape) == 2:
		blockheight, blockwidth = blockshape
		return permuteToBlocks2d(arr, blockheight, blockwidth)
	elif len(blockshape) == 3:
		blockdepth, blockheight, blockwidth = blockshape
		return permuteToBlocks3d(arr, blockdepth, blockheight, blockwidth)
	else:
		raise NotImplementedError("Only for dimensions 2 and 3.")


def permuteToBlocks3d(arr, blockdepth, blockheight, blockwidth):
	depth, height, width = arr.shape
	arr_ = arr.reshape(height * depth, width)
	arr_ = permuteToBlocks2d(arr_, blockheight, blockwidth)
	arr_.shape = depth, height * width
	return permuteToBlocks2d(arr_, blockdepth, blockwidth * blockheight)


def permuteToBlocks2d(arr, blockheight, blockwidth):
	_height, width = arr.shape
	arr = arr.flatten()
	new = zeros(size(arr))
	for i in range(size(arr)):
		blockx = (i % width) / blockwidth
		blocky = i / width / blockheight
		blockoffset = blocky * width / blockwidth + blockx
		blockoffset *= blockwidth * blockheight
		inblockx = i % blockwidth
		inblocky = (i / width) % blockheight
		j = blockoffset + inblocky * blockwidth + inblockx
		new[j] = arr[i]
	return new


def triu2flat(m):
	""" Flattens an upper triangular matrix, returning a vector of the
	non-zero elements. """
	dim = m.shape[0]
	res = zeros(dim * (dim + 1) / 2)
	index = 0
	for row in range(dim):
		res[index:index + dim - row] = m[row, row:]
		index += dim - row
	return res


def flat2triu(a, dim):
	""" Produces an upper triangular matrix of dimension dim from the elements of the given vector. """
	res = zeros((dim, dim))
	index = 0
	for row in range(dim):
		res[row, row:] = a[index:index + dim - row]
		index += dim - row
	return res


def blockList2Matrix(l):
	""" Converts a list of matrices into a corresponding big block-diagonal one. """
	dims = [m.shape[0] for m in l]
	s = sum(dims)
	res = zeros((s, s))
	index = 0
	for i in range(len(l)):
		d = dims[i]
		m = l[i]
		res[index:index + d, index:index + d] = m
		index += d
	return res


def blockCombine(l):
	""" Produce a matrix from a list of lists of its components. """
	l = [list(map(mat, row)) for row in l]
	hdims = [m.shape[1] for m in l[0]]
	hs = sum(hdims)
	vdims = [row[0].shape[0] for row in l]
	vs = sum(vdims)
	res = zeros((hs, vs))
	vindex = 0
	for i, row in enumerate(l):
		hindex = 0
		for j, m in enumerate(row):
			res[vindex:vindex + vdims[i], hindex:hindex + hdims[j]] = m
			hindex += hdims[j]
		vindex += vdims[i]
	return res


def avgFoundAfter(decreasingTargetValues, listsOfActualValues, batchSize=1, useMedian=False):
	""" Determine the average number of steps to reach a certain value (for the first time),
	given a list of value sequences.
	If a value is not always encountered, the length of the longest sequence is used.
	Returns an array. """
	from scipy import sum
	numLists = len(listsOfActualValues)
	longest = max(list(map(len, listsOfActualValues)))
	# gather a list of indices of first encounters
	res = [[0] for _ in range(numLists)]
	for tval in decreasingTargetValues:
		for li, l in enumerate(listsOfActualValues):
			lres = res[li]
			found = False
			for i in range(lres[-1], len(l)):
				if l[i] <= tval:
					lres.append(i)
					found = True
					break
			if not found:
				lres.append(longest)
	tmp = array(res)
	if useMedian:
		resx = median(tmp, axis=0)[1:]
	else:
		resx = sum(tmp, axis=0)[1:] / float(numLists)
	return resx * batchSize


class DivergenceError(Exception):
	""" Raised when an algorithm diverges. """


def matchingDict(d, selection, require_existence=False):
	""" Determines if the dictionary d conforms to the specified selection,
	i.e. if a (key, x) is in the selection, then if key is in d as well it must be x
	or contained in x (if x is a list). """
	for k, v in list(selection.items()):
		if k in d:
			if isinstance(v, list):
				if d[k] not in v:
					return False
			else:
				if d[k] != v:
					return False
		elif require_existence:
			return False
	return True


def subDict(d, allowedkeys, flip=False):
	""" Returns a new dictionary with a subset of the entries of d
	that have on of the (dis-)allowed keys."""
	res = {}
	for k, v in list(d.items()):
		if (k in allowedkeys) ^ flip:
			res[k] = v
	return res


def dictCombinations(listdict):
	""" Iterates over dictionaries that go through every possible combination
	of key-value pairs as specified in the lists of values for each key in listdict."""
	listdict = listdict.copy()
	if len(listdict) == 0:
		return [{}]
	k, vs = listdict.popitem()
	res = dictCombinations(listdict)
	if isinstance(vs, list) or isinstance(vs, tuple):
		res = [dict(d, **{k:v}) for d in res for v in sorted(set(vs))]
	else:
		res = [dict(d, **{k:vs}) for d in res]
	return res


def r_argmax(v):
	""" Acts like scipy argmax, but break ties randomly. """
	if len(v) == 1:
		return 0
	maxbid = max(v)
	maxbidders = [i for (i, b) in enumerate(v) if b==maxbid]
	return choice(maxbidders)

def all_argmax(x):
	""" Return the indices of all values that are equal to the maximum: no breaking ties. """
	m = max(x)
	return [i for i, v in enumerate(x) if v == m]

def dense_orth(dim):
	""" Constructs a dense orthogonal matrix. """
	from scipy import rand
	from scipy.linalg import orth
	return orth(rand(dim, dim))

def sparse_orth(d):
	""" Constructs a sparse orthogonal matrix.

	The method is described in:
	Gi-Sang Cheon et al., Constructions for the sparsest orthogonal matrices,
	Bull. Korean Math. Soc 36 (1999) No.1 pp.199-129
	"""
	from scipy.sparse import eye
	from scipy import r_, pi, sin, cos
	if d%2 == 0:
		seq = r_[0:d:2,1:d-1:2]
	else:
		seq = r_[0:d-1:2,1:d:2]
	Q = eye(d,d).tocsc()
	for i in seq:
		theta = random() * 2 * pi
		flip = (random() - 0.5)>0;
		Qi = eye(d,d).tocsc()
		Qi[i,i] = cos(theta)
		Qi[(i+1),i] = sin(theta)
		if flip > 0:
			Qi[i,(i+1)] = -sin(theta)
			Qi[(i+1),(i+1)] = cos(theta)
		else:
			Qi[i,(i+1)] = sin(theta)
			Qi[(i+1),(i+1)] = -cos(theta)
		Q = Q*Qi;
	return Q

def xhash(arr):
	""" Hashing function for arrays. Use with care. """
	import hashlib
	return hashlib.sha1(arr).hexdigest()

def binArr2int(arr):
	""" Convert a binary array into its (long) integer representation. """
	from numpy import packbits
	tmp2 = packbits(arr.astype(int))
	return sum(val * 256 ** i for i, val in enumerate(tmp2[::-1]))

def uniqueArrays(vs):
	""" create a set of arrays """
	resdic = {}
	for v in vs:
		resdic[xhash(v)] = v
	return list(resdic.values())


def seedit(seed=0):
	""" Fixed seed makes for repeatability, but there may be two different
	random number generators involved. """
	import random
	import numpy
	random.seed(seed)
	numpy.random.seed(seed)



def weightedUtest(g1, w1, g2, w2):
	""" Determines the confidence level of the assertion:
	'The values of g2 are higher than those of g1'.
	(adapted from the scipy.stats version)

	Twist: here the elements of each group have associated weights,
	corresponding to how often they are present (i.e. two identical entries with
	weight w are equivalent to one entry with weight 2w).
	Reference: "Studies in Continuous Black-box Optimization", Schaul, 2011 [appendix B].

	TODO: make more efficient for large sets.
	"""
	from scipy.stats.distributions import norm
	import numpy
	n1 = sum(w1)
	n2 = sum(w2)
	u1 = 0.
	for x1, wx1 in zip(g1, w1):
		for x2, wx2 in zip(g2, w2):
			if x1 == x2:
				u1 += 0.5 * wx1 * wx2
			elif x1 > x2:
				u1 += wx1 * wx2
	mu = n1*n2/2.
	sigu = numpy.sqrt(n1*n2*(n1+n2+1)/12.)
	z = (u1 - mu) / sigu
	conf = norm.cdf(z)
	return conf



__author__ = 'Mehmet Mert Yildiran, mert.yildiran@bil.omu.edu.tr'

import datetime # Supplies classes for manipulating dates and times in both simple and complex ways
import os.path # The path module suitable for the operating system Python is running on, and therefore usable for local paths

# Pair class
class Pair(object):
	def __init__(self, timestamp1, timestamp2, direction): # Initialize the object
		self.timestamp1 = timestamp1 # Memory starting time of a sense
		self.timestamp2 = timestamp2 # Memory starting time of another sense
		self.direction = direction # Direction. For example H2V, V2H, H2H etc.

# Convert object to dictionary
def makeit_dict(obj):
	if isinstance(obj, set):
		return list(obj)
	return obj.__dict__

class MapperUtil():

	# Write a pair function
	@staticmethod
	def write_pair(timestamp1, timestamp2, direction):
		PR_FILE_PATH = os.path.expanduser("~/Hippocampus/crossmodal/mappings/" +  str(datetime.date.today()) + ".pr") # Path for pairs file

		pair = Pair(timestamp1, timestamp2, direction) # Create an object from Pair class
		mode = 'a' if os.path.exists(PR_FILE_PATH) else 'w' # If pairs file exist, file open mode will be append(a) else write(w)
		with open(PR_FILE_PATH, mode) as pr_file: # Open file
			pr_file.write(str(makeit_dict(pair)) + '\n') # Write pair with only one line

	# Read a pair function
	@staticmethod
	def read_pair(date_day,nth_record):
		PR_FILE_PATH = os.path.expanduser("~/Hippocampus/crossmodal/mappings/" +  date_day + ".pr") # Path for pairs file

		if os.path.exists(PR_FILE_PATH): # If pairs file exist
			with open(PR_FILE_PATH, 'r') as pr_file: # Open file
				pair = eval(pr_file.readlines()[nth_record]) # Evaluate the line, which will return a dictionary
				return pair # Return pair to call
		else: # If pairs file doesn't exist
			#raise ValueError('PR file doesn\'t exist!') # Raise a ValueError
			return False

	# Get pairs function
	@staticmethod
	def get_pairs(date_day,from_line=0):
		PR_FILE_PATH = os.path.expanduser("~/Hippocampus/crossmodal/mappings/" +  date_day + ".pr") # Path for pairs file
		pairs = []
		if os.path.exists(PR_FILE_PATH): # If pairs file exist
			with open(PR_FILE_PATH, 'r') as pr_file: # Open file
				for line in pr_file.readlines()[from_line:]: # Get whole lines starting from that line, default zero
					pair = eval(line) # Evaluate the line, which will return a dictionary
					pairs.append(pair) # Append pair to list in order
				return pairs # Return pair list to call
		else: # If pairs file doesn't exist
			#raise ValueError('PR file doesn\'t exist!') # Raise a ValueError
			return False



import random
import pickle
from itertools import chain
from scipy import zeros, resize, ravel, asarray
import scipy

from cerebrum.neuralnet.utilities import Serializable

class OutOfSyncError(Exception): pass
class VectorFormatError(Exception): pass
class NoLinkedFieldsError(Exception): pass


class DataSet(Serializable):
	"""DataSet is a general base class for other data set classes
	(e.g. SupervisedDataSet, SequentialDataSet, ...). It consists of several
	fields. A field is a NumPy array with a label (a string) attached to it.
	Fields can be linked together which means they must have the same length."""

	def __init__(self):
		self.data = {}
		self.endmarker = {}
		self.link = []
		self.index = 0

		# row vectors returned by getLinked can have different formats:
		# '1d'       example: array([1, 2, 3])
		# '2d'       example: array([[1, 2, 3]])
		# 'list'     example: [1, 2, 3]
		self.vectorformat = 'none'

	def __str__(self):
		"""Return a string representation of a dataset."""
		s = ""
		for key in self.data:
			s = s + key + ": dim" + str(self.data[key].shape) + "\n" + str(self.data[key][:self.endmarker[key]]) + "\n\n"
		return s

	def __getitem__(self, field):
		"""Return the given field."""
		return self.getField(field)

	def __iter__(self):
		self.reset()
		while not self.endOfData():
			yield self.getLinked()

	def getVectorFormat(self):
		"""Returns the current vector format."""
		return self.__vectorformat

	def setVectorFormat(self, vf):
		"""Determine which format to use for returning vectors. Use the property vectorformat.

			:key type: possible types are '1d', '2d', 'list'
				  '1d' - example: array([1,2,3])
				  '2d' - example: array([[1,2,3]])
				'list' - example: [1,2,3]
				'none' - no conversion
		 """
		switch = {
			'1d': self._convertArray1d,
			'2d': self._convertArray2d,
			'list': self._convertList,
			'none': lambda x:x
		}
		try:
			self._convert = switch[vf]
			self.__vectorformat = vf
		except KeyError:
			raise VectorFormatError("vector format must be one of '1d', '2d', 'list'. given: %s" % vf)

	vectorformat = property(getVectorFormat, setVectorFormat, None, "vectorformat can be '1d', '2d' or 'list'")

	def _convertList(self, vector):
		"""Converts the incoming vector to a python list."""
		return ravel(vector).tolist()

	def _convertArray1d(self, vector):
		"""Converts the incoming vector to a 1d vector with shape (x,) where x
		is the number of elements."""
		return ravel(vector)

	def _convertArray2d(self, vector, column=False):
		"""Converts the incoming `vector` to a 2d vector with shape (1,x), or
		(x,1) if `column` is set, where x is the number of elements."""
		a = asarray(vector)
		sh = a.shape
		# also reshape scalar values to 2d-index
		if len(sh) == 0:
			sh = (1,)
		if len(sh) == 1:
			# use reshape to add extra dimension
			if column:
				return a.reshape((sh[0], 1))
			else:
				return a.reshape((1, sh[0]))
		else:
			# vector is not 1d, return a without change
			return a

	def addField(self, label, dim):
		"""Add a field to the dataset.

		A field consists of a string `label`  and a numpy ndarray of dimension
		`dim`."""
		self.data[label] = zeros((0, dim), float)
		self.endmarker[label] = 0

	def setField(self, label, arr):
		"""Set the given array `arr` as the new array of field `label`,"""
		as_arr = asarray(arr)
		self.data[label] = as_arr
		self.endmarker[label] = as_arr.shape[0]

	def linkFields(self, linklist):
		"""Link the length of several fields given by the list of strings
		`linklist`."""
		length = self[linklist[0]].shape[0]
		for l in linklist:
			if self[l].shape[0] != length:
				raise OutOfSyncError
		self.link = linklist

	def unlinkFields(self, unlinklist=None):
		"""Remove fields from the link list or clears link given by the list of
		string `linklist`.

		This method has no effect if fields are not linked."""
		link = self.link
		if unlinklist is not None:
			for l in unlinklist:
				if l in self.link:
					link.remove(l)
			self.link = link
		else:
			self.link = []

	def getDimension(self, label):
		"""Return the dimension/number of columns for the field given by
		`label`."""
		try:
			dim = self.data[label].shape[1]
		except KeyError:
			raise KeyError('dataset field %s not found.' % label)
		return dim

	def __len__(self):
		"""Return the length of the linked data fields. If no linked fields exist,
		return the length of the longest field."""
		return self.getLength()

	def getLength(self):
		"""Return the length of the linked data fields. If no linked fields exist,
		return the length of the longest field."""
		if self.link == []:
			try:
				length = self.endmarker[max(self.endmarker)]
			except ValueError:
				return 0
			return length
		else:
			# all linked fields have equal length. return the length of the first.
			l = self.link[0]
			return self.endmarker[l]

	def _resize(self, label=None):
		if label:
			label = [label]
		elif self.link:
			label = self.link
		else:
			label = self.data

		for l in label:
			self.data[l] = self._resizeArray(self.data[l])

	def _resizeArray(self, a):
		"""Increase the buffer size. It should always be one longer than the
		current sequence length and double on every growth step."""
		shape = list(a.shape)
		shape[0] = (shape[0] + 1) * 2
		return resize(a, shape)

	def _appendUnlinked(self, label, row):
		"""Append `row` to the field array with the given `label`.

		Do not call this function from outside, use ,append() instead.
		Automatically casts vector to a 2d (or higher) shape."""
		if self.data[label].shape[0] <= self.endmarker[label]:
			self._resize(label)

		self.data[label][self.endmarker[label], :] = row
		self.endmarker[label] += 1

	def append(self, label, row):
		"""Append `row` to the array given by `label`.

		If the field is linked with others, the function throws an
		`OutOfSyncError` because all linked fields always have to have the same
		length. If you want to add a row to all linked fields, use appendLink
		instead."""
		if label in self.link:
			raise OutOfSyncError
		self._appendUnlinked(label, row)

	def appendLinked(self, *args):
		"""Add rows to all linked fields at once."""
		assert len(args) == len(self.link)
		for i, l in enumerate(self.link):
			self._appendUnlinked(l, args[i])

	def getLinked(self, index=None):
		"""Access the dataset randomly or sequential.

		If called with `index`, the appropriate line consisting of all linked
		fields is returned and the internal marker is set to the next line.
		Otherwise the marked line is returned and the marker is moved to the
		next line."""
		if self.link == []:
			raise NoLinkedFieldsError('The dataset does not have any linked fields.')

		if index == None:
			# no index given, return the currently marked line and step marker one line forward
			index = self.index
			self.index += 1
		else:
			# return the indexed line and move marker to next line
			self.index = index + 1
		if index >= self.getLength():
			raise IndexError('index out of bounds of the dataset.')

		return [self._convert(self.data[l][index]) for l in self.link]

	def getField(self, label):
		"""Return the entire field given by `label` as an array or list,
		depending on user settings."""
		# Note: label_data should always be a np.array, so this will never
		# actually clone a list (performances are O(1)).
		label_data = self.data[label][:self.endmarker[label]]

		# Convert to list if requested.
		if self.vectorformat == 'list':
			label_data = label_data.tolist()

		return label_data

	def hasField(self, label):
		"""Tell whether the field given by `label` exists."""
		return label in self.data

	def getFieldNames(self):
		"""Return the names of the currently defined fields."""
		return list(self.data.keys())

	def convertField(self, label, newtype):
		"""Convert the given field to a different data type."""
		try:
			self.setField(label, self.data[label].astype(newtype))
		except KeyError:
			raise KeyError('convertField: dataset field %s not found.' % label)

	def endOfData(self):
		"""Tell if the end of the data set is reached."""
		return self.index == self.getLength()

	def reset(self):
		"""Reset the marker to the first line."""
		self.index = 0

	def clear(self, unlinked=False):
		"""Clear the dataset.

		If linked fields exist, only the linked fields will be deleted unless
		`unlinked` is set to True. If no fields are linked, all data will be
		deleted."""
		self.reset()
		keys = self.link
		if keys == [] or unlinked:
			# iterate over all fields instead
			keys = self.data

		for k in keys:
			shape = list(self.data[k].shape)
			# set to zero rows
			shape[0] = 0
			self.data[k] = zeros(shape)
			self.endmarker[k] = 0

	@classmethod
	def reconstruct(cls, filename):
		"""Read an incomplete data set (option arraysonly) into the given one. """
		# FIXME: Obsolete! Kept here because of some old files...
		obj = cls(1, 1)
		for key, val in pickle.load(file(filename)).items():
			obj.setField(key, val)
		return obj

	def save_pickle(self, flo, protocol=0, compact=False):
		"""Save data set as pickle, removing empty space if desired."""
		if compact:
			# remove padding of zeros for each field
			for field in self.getFieldNames():
				temp = self[field][0:self.endmarker[field] + 1, :]
				self.setField(field, temp)
		Serializable.save_pickle(self, flo, protocol)

	def __reduce__(self):
		def creator():
			obj = self.__class__()
			obj.vectorformat = self.vectorformat
			return obj
		args = tuple()
		state = {
			'data': self.data,
			'link': self.link,
			'endmarker': self.endmarker,
		}
		return creator, args, state, iter([]), iter({})

	def copy(self):
		"""Return a deep copy."""
		import copy
		return copy.deepcopy(self)

	def batches(self, label, n, permutation=None):
		"""Yield batches of the size of n from the dataset.

		A single batch is an array of with dim columns and n rows. The last
		batch is possibly smaller.

		If permutation is given, batches are yielded in the corresponding
		order."""
		# First calculate how many batches we will have
		full_batches, rest = divmod(len(self), n)
		number_of_batches = full_batches if rest == 0 else full_batches + 1

		# We make one iterator for the startindexes ...
		startindexes = (i * n for i in range(number_of_batches))
		# ... and one for the stop indexes
		stopindexes = (((i + 1) * n) for i in range(number_of_batches - 1))
		# The last stop index is the last element of the list (last batch
		# might not be filled completely)
		stopindexes = chain(stopindexes, [len(self)])
		# Now combine them
		indexes = list(zip(startindexes, stopindexes))

		# Shuffle them according to the permutation if one is given
		if permutation is not None:
			indexes = [indexes[i] for i in permutation]

		for start, stop in indexes:
			yield self.data[label][start:stop]

	def randomBatches(self, label, n):
		"""Like .batches(), but the order is random."""
		permutation = random.shuffle(list(range(len(self))))
		return self.batches(label, n, permutation)

	def replaceNansByMeans(self):
		"""Replace all not-a-number entries in the dataset by the means of the
		corresponding column."""
		for d in self.data.values():
			means = scipy.nansum(d[:self.getLength()], axis=0) / self.getLength()
			for i in range(self.getLength()):
				for j in range(d.dim):
					if not scipy.isfinite(d[i, j]):
						d[i, j] = means[j]



from __future__ import print_function

from numpy import random
from random import sample
from scipy import isscalar

from cerebrum.neuralnet.datasets.dataset import DataSet
from cerebrum.neuralnet.utilities import fListToString


class SupervisedDataSet(DataSet):
	"""SupervisedDataSets have two fields, one for input and one for the target.
	"""

	def __init__(self, inp, target):
		"""Initialize an empty supervised dataset.

		Pass `inp` and `target` to specify the dimensions of the input and
		target vectors."""
		DataSet.__init__(self)
		if isscalar(inp):
			# add input and target fields and link them
			self.addField('input', inp)
			self.addField('target', target)
		else:
			self.setField('input', inp)
			self.setField('target', target)

		self.linkFields(['input', 'target'])

		# reset the index marker
		self.index = 0

		# the input and target dimensions
		self.indim = self.getDimension('input')
		self.outdim = self.getDimension('target')

	def __reduce__(self):
		_, _, state, _, _ = super(SupervisedDataSet, self).__reduce__()
		creator = self.__class__
		args = self.indim, self.outdim
		return creator, args, state, iter([]), iter({})

	def addSample(self, inp, target):
		"""Add a new sample consisting of `input` and `target`."""
		self.appendLinked(inp, target)

	def getSample(self, index=None):
		"""Return a sample at `index` or the current sample."""
		return self.getLinked(index)

	def setField(self, label, arr, **kwargs):
		"""Set the given array `arr` as the new array of the field specfied by
		`label`."""
		DataSet.setField(self, label, arr, **kwargs)
		# refresh dimensions, in case any of these fields were modified
		if label == 'input':
			self.indim = self.getDimension('input')
		elif label == 'target':
			self.outdim = self.getDimension('target')

	def _provideSequences(self):
		"""Return an iterator over sequence lists, although the dataset contains
		only single samples."""
		return iter([[x] for x in iter(self)])

	def evaluateMSE(self, f, **args):
		"""Evaluate the predictions of a function on the dataset and return the
		Mean Squared Error, incorporating importance."""
		ponderation = 0.
		totalError = 0
		for seq in self._provideSequences():
			e, p = self._evaluateSequence(f, seq, **args)
			totalError += e
			ponderation += p
		assert ponderation > 0
		return totalError/ponderation

	def _evaluateSequence(self, f, seq, verbose = False):
		"""Return the ponderated MSE over one sequence."""
		totalError = 0.
		ponderation = 0.
		for input, target in seq:
			res = f(input)
			e = 0.5 * sum((target-res).flatten()**2)
			totalError += e
			ponderation += len(target)
			if verbose:
				print((    'out:    ', fListToString( list( res ) )))
				print((    'correct:', fListToString( target )))
				print((    'error: % .8f' % e))
		return totalError, ponderation

	def evaluateModuleMSE(self, module, averageOver = 1, **args):
		"""Evaluate the predictions of a module on a dataset and return the MSE
		(potentially average over a number of epochs)."""
		res = 0.
		for dummy in range(averageOver):
			module.reset()
			res += self.evaluateMSE(module.activate, **args)
		return res/averageOver

	def splitWithProportion(self, proportion = 0.5):
		"""Produce two new datasets, the first one containing the fraction given
		by `proportion` of the samples."""
		indicies = random.permutation(len(self))
		separator = int(len(self) * proportion)

		leftIndicies = indicies[:separator]
		rightIndicies = indicies[separator:]

		leftDs = SupervisedDataSet(inp=self['input'][leftIndicies].copy(),
								   target=self['target'][leftIndicies].copy())
		rightDs = SupervisedDataSet(inp=self['input'][rightIndicies].copy(),
									target=self['target'][rightIndicies].copy())
		return leftDs, rightDs



__author__ = 'Mehmet Mert Yildiran, mert.yildiran@bil.omu.edu.tr'

import datetime # Supplies classes for manipulating dates and times in both simple and complex ways
import os.path # The path module suitable for the operating system Python is running on, and therefore usable for local paths

# Memory class
class Memory(object):
	def __init__(self, starting_time, ending_time, thresh_binary, frame_delta_colored): # Initialize the object
		self.starting_time = starting_time # Starting time attribute
		self.ending_time = ending_time # Ending time attribute
		self.thresh_binary = thresh_binary # Thresh binary frame attribute
		self.frame_delta_colored = frame_delta_colored # Frame delta colored frame attribute

# Timestamp class
class Timestamp(object):
	def __init__(self, starting_time, ending_time): # Initialize the object
		self.starting_time = starting_time # Starting time attribute
		self.ending_time = ending_time # Ending time attribute

# Convert object to dictionary
def makeit_dict(obj):
	if isinstance(obj, set):
		return list(obj)
	return obj.__dict__

class VisionMemoryUtil():

	# Write a memory function
	@staticmethod
	def write_memory(thresh_binary, frame_delta_colored, starting_time, ending_time):
		MEM_FILE_PATH = os.path.expanduser("~/Hippocampus/vision/memory/" +  str(datetime.date.today()) + ".mem") # Path for mem file
		TSTP_FILE_PATH = os.path.expanduser("~/Hippocampus/vision/memory/" +  str(datetime.date.today()) + ".tstp") # Path for tstp file

		memory = Memory(starting_time.strftime("%Y-%m-%d %H:%M:%S.%f"), ending_time.strftime("%Y-%m-%d %H:%M:%S.%f"), thresh_binary, frame_delta_colored) # Create an object from Memory class
		mode = 'a' if os.path.exists(MEM_FILE_PATH) else 'w' # If memory file exist, file open mode will be append(a) else write(w)
		with open(MEM_FILE_PATH, mode) as mem_file: # Open file
			mem_file.write(str(makeit_dict(memory)) + '\n') # Write memory in only one line

		timestamp = Timestamp(starting_time.strftime("%Y-%m-%d %H:%M:%S.%f"), ending_time.strftime("%Y-%m-%d %H:%M:%S.%f")) # Create an object from Timestamp class
		mode = 'a' if os.path.exists(TSTP_FILE_PATH) else 'w' # If timestamp file exist, file open mode will be append(a) else write(w)
		with open(TSTP_FILE_PATH, mode) as tstp_file: # Open file
			tstp_file.write(str(makeit_dict(timestamp)) + '\n') # Write timestamp in only one line

	# Read a memory function
	@staticmethod
	def read_memory(date_day,starting_time):
		MEM_FILE_PATH = os.path.expanduser("~/Hippocampus/vision/memory/" +  date_day + ".mem") # Path for mem file
		memory_list = []
		if os.path.exists(MEM_FILE_PATH): # If memory file exist
			with open(MEM_FILE_PATH, 'r') as mem_file: # Open file
				for line in mem_file.readlines(): # Get whole lines
					memory = eval(line) # Evaluate the line, which will return a dictionary
					if memory['starting_time'] == starting_time: # If current memory's starting time is equal to function's parameter
						return memory # Return current memory to call
				return False # Else return False
		else: # If memory file doesn't exist
			raise ValueError('MEM file doesn\'t exist!') # Raise a ValueError

	# Read timestamps function
	@staticmethod
	def read_timestamps(date_day,from_line=0):
		TSTP_FILE_PATH = os.path.expanduser("~/Hippocampus/vision/memory/" +  date_day + ".tstp") # Path for tstp file
		timestamp_list = []
		if os.path.exists(TSTP_FILE_PATH): # If timestamp file exist
			with open(TSTP_FILE_PATH, 'r') as tstp_file: # Open file
				for line in tstp_file.readlines()[from_line:]: # Get whole lines starting from that line, default zero
					timestamp = eval(line) # Evaluate the line, which will return a dictionary
					timestamp_list.append(timestamp) # Append timestamp to list in order
				return timestamp_list # Return timestamp list to call
		else: # If timestamp file doesn't exist
			return False

# Example USAGE block. NOT FUNCTIONAL
if __name__ == "__main__":
	timestamp_list = VisionMemoryUtil.read_timestamps(str(datetime.date.today()))
	#for timestamp in timestamp_list:
		#print "--------------------------"
		#print timestamp['starting_time']
		#print timestamp['ending_time']
	print len(timestamp_list)
	memory = VisionMemoryUtil.read_memory(str(datetime.date.today()), timestamp_list[-3]['starting_time'])
	print len(memory['thresh_binary'])
	print len(memory['frame_delta_colored'])

	#CHUNK = 1024
	#WIDTH = 2
	#CHANNELS = 2
	#RATE = 44100
	#p = pyaudio.PyAudio()
	#stream = p.open(format=p.get_format_from_width(WIDTH), channels=CHANNELS, rate=RATE, output=True, frames_per_buffer=CHUNK)
	#for data in memory['data']:
		#stream.write(data)



__author__ = 'Mehmet Mert Yildiran, mert.yildiran@bil.omu.edu.tr'

import datetime
import os.path
import os

HEARING_MEM_FILE_PATH = os.path.expanduser("~/Hippocampus/hearing/memory/" +  str(datetime.date.today()) + ".mem") # Path for mem file
HEARING_TSTP_FILE_PATH = os.path.expanduser("~/Hippocampus/hearing/memory/" +  str(datetime.date.today()) + ".tstp") # Path for tstp file
VISION_MEM_FILE_PATH = os.path.expanduser("~/Hippocampus/vision/memory/" +  str(datetime.date.today()) + ".mem") # Path for mem file
VISION_TSTP_FILE_PATH = os.path.expanduser("~/Hippocampus/vision/memory/" +  str(datetime.date.today()) + ".tstp") # Path for tstp file
LANGUAGE_MEM_FILE_PATH = os.path.expanduser("~/Hippocampus/language/memory/" +  str(datetime.date.today()) + ".mem") # Path for mem file
LANGUAGE_TSTP_FILE_PATH = os.path.expanduser("~/Hippocampus/language/memory/" +  str(datetime.date.today()) + ".tstp") # Path for tstp file
PR_FILE_PATH = os.path.expanduser("~/Hippocampus/crossmodal/mappings/" +  str(datetime.date.today()) + ".pr") # Path for pairs file

if os.path.exists(HEARING_MEM_FILE_PATH):
	os.remove(HEARING_MEM_FILE_PATH)
	print HEARING_MEM_FILE_PATH + " is removed"
if os.path.exists(HEARING_TSTP_FILE_PATH):
	os.remove(HEARING_TSTP_FILE_PATH)
	print HEARING_TSTP_FILE_PATH + " is removed"
if os.path.exists(VISION_MEM_FILE_PATH):
	os.remove(VISION_MEM_FILE_PATH)
	print VISION_MEM_FILE_PATH + " is removed"
if os.path.exists(VISION_TSTP_FILE_PATH):
	os.remove(VISION_TSTP_FILE_PATH)
	print VISION_TSTP_FILE_PATH + " is removed"
if os.path.exists(LANGUAGE_MEM_FILE_PATH):
	os.remove(LANGUAGE_MEM_FILE_PATH)
	print LANGUAGE_MEM_FILE_PATH + " is removed"
if os.path.exists(LANGUAGE_TSTP_FILE_PATH):
	os.remove(LANGUAGE_TSTP_FILE_PATH)
	print LANGUAGE_TSTP_FILE_PATH + " is removed"
if os.path.exists(PR_FILE_PATH):
	os.remove(PR_FILE_PATH)
	print PR_FILE_PATH + " is removed"



from scipy import ravel, r_
from random import sample

from cerebrum.neuralnet.datasets.supervised import SupervisedDataSet


class EmptySequenceError(Exception): pass


class SequentialDataSet(SupervisedDataSet):
	"""A SequentialDataSet is like a SupervisedDataSet except that it can keep
	track of sequences of samples. Indices of a new sequence are stored whenever
	the method newSequence() is called. The last (open) sequence is considered
	a normal sequence even though it does not have a following "new sequence"
	marker."""

	def __init__(self, indim, targetdim):
		SupervisedDataSet.__init__(self, indim, targetdim)
		# add field that stores the beginning of a new episode
		self.addField('sequence_index', 1)
		self.append('sequence_index', 0)
		self.currentSeq = 0

	def newSequence(self):
		"""Marks the beginning of a new sequence. this function does nothing if
		called at the very start of the data set. Otherwise, it starts a new
		sequence. Empty sequences are not allowed, and an EmptySequenceError
		exception will be raised."""
		length = self.getLength()
		if length != 0:
			if ravel(self.getField('sequence_index'))[-1] == length:
				raise EmptySequenceError
			self._appendUnlinked('sequence_index', length)

	def _getSequenceField(self, index, field):
		"""Return a sequence of one single field given by `field` and indexed by
		`index`."""
		seq = ravel(self.getField('sequence_index'))
		if len(seq) == index + 1:
			# user wants to access the last sequence, return until end of data
			return self.getField(field)[seq[index]:]
		if len(seq) < index + 1:
			# sequence index beyond number of sequences. raise exception
			raise IndexError('sequence does not exist.')
		return self.getField(field)[seq[index]:seq[index + 1]]

	def getSequence(self, index):
		"""Returns the sequence given by `index`.

		A list of arrays is returned for the linked arrays. It is assumed that
		the last sequence goes until the end of the dataset."""
		return [self._getSequenceField(index, l) for l in self.link]

	def getSequenceIterator(self, index):
		"""Return an iterator over the samples of the sequence specified by
		`index`.

		Each element is a tuple."""
		return zip(*self.getSequence(index))

	def endOfSequence(self, index):
		"""Return True if the marker was moved over the last element of
		sequence `index`, False otherwise.

		Mostly used like .endOfData() with while loops."""
		seq = ravel(self.getField('sequence_index'))
		if len(seq) == index + 1:
			# user wants to access the last sequence, return until end of data
			return self.endOfData()
		if len(seq) < index + 1:
			# sequence index beyond number of sequences. raise exception
			raise IndexError('sequence does not exist.')
		else:
			return self.index >= seq[index + 1]

	def gotoSequence(self, index):
		"""Move the internal marker to the beginning of sequence `index`."""
		try:
			self.index = ravel(self.getField('sequence_index'))[index]
		except IndexError:
			raise IndexError('sequence does not exist')

	def getCurrentSequence(self):
		"""Return the current sequence, according to the marker position."""
		seq = ravel(self.getField('sequence_index'))
		return len(seq) - sum(seq > self.index) - 1

	def getNumSequences(self):
		"""Return the number of sequences. The last (open) sequence is also
		counted in, even though there is no additional 'newSequence' marker."""
		return self.getField('sequence_index').shape[0]

	def getSequenceLength(self, index):
		"""Return the length of the given sequence. If `index` is pointing
		to the last sequence, the sequence is considered to go until the end
		of the dataset."""
		seq = ravel(self.getField('sequence_index'))
		if len(seq) == index + 1:
			# user wants to access the last sequence, return until end of data
			return int(self.getLength() - seq[index])
		if len(seq) < index + 1:
			# sequence index beyond number of sequences. raise exception
			raise IndexError('sequence does not exist.')
		return int(seq[index + 1] - seq[index])

	def removeSequence(self, index):
		"""Remove the `index`'th sequence from the dataset and places the
		marker to the sample following the removed sequence."""
		if index >= self.getNumSequences():
			# sequence doesn't exist, raise exception
			raise IndexError('sequence does not exist.')
		sequences = ravel(self.getField('sequence_index'))
		seqstart = sequences[index]
		if index == self.getNumSequences() - 1:
			# last sequence is going to be removed
			lastSeqDeleted = True
			seqend = self.getLength()
		else:
			lastSeqDeleted = False
			# sequence to remove is not last one (sequence_index exists)
			seqend = sequences[index + 1]

		# cut out data from all fields
		for label in self.link:
			# concatenate rows from start to seqstart and from seqend to end
			self.data[label] = r_[self.data[label][:seqstart, :], self.data[label][seqend:, :]]
			# update endmarkers of linked fields
			self.endmarker[label] -= seqend - seqstart

		# update sequence indices
		for i, val in enumerate(sequences):
			if val > seqstart:
				self.data['sequence_index'][i, :] -= seqend - seqstart

		# remove sequence index of deleted sequence and reduce its endmarker
		self.data['sequence_index'] = r_[self.data['sequence_index'][:index, :], self.data['sequence_index'][index + 1:, :]]
		self.endmarker['sequence_index'] -= 1

		if lastSeqDeleted:
			# last sequence was removed
			# move sequence marker to last remaining sequence
			self.currentSeq = index - 1
			# move sample marker to end of dataset
			self.index = self.getLength()
			# if there was only 1 sequence left, re-initialize sequence index
			if self.getLength() == 0:
				self.clear()
		else:
			# removed sequence was not last one (sequence_index exists)
			# move sequence marker to the new sequence at position 'index'
			self.currentSeq = index
			# move sample marker to beginning of sequence at position 'index'
			self.index = ravel(self.getField('sequence_index'))[index]


	def clear(self):
		SupervisedDataSet.clear(self, True)
		self._appendUnlinked('sequence_index', [0])
		self.currentSeq = 0

	def __iter__(self):
		"""Create an iterator object over sequences which are themselves
		iterable objects."""
		for i in range(self.getNumSequences()):
			yield self.getSequenceIterator(i)

	def _provideSequences(self):
		"""Return an iterator over sequence lists."""
		return iter(map(list, iter(self)))

	def evaluateModuleMSE(self, module, averageOver=1, **args):
		"""Evaluate the predictions of a module on a sequential dataset
		and return the MSE (potentially average over a number of epochs)."""
		res = 0.
		for dummy in range(averageOver):
			ponderation = 0.
			totalError = 0
			for seq in self._provideSequences():
				module.reset()
				e, p = self._evaluateSequence(module.activate, seq, **args)
				totalError += e
				ponderation += p
			assert ponderation > 0
			res += totalError / ponderation
		return res / averageOver

	def splitWithProportion(self, proportion=0.5):
		"""Produce two new datasets, each containing a part of the sequences.

		The first dataset will have a fraction given by `proportion` of the
		dataset."""
		l = self.getNumSequences()
		leftIndices = sample(list(range(l)), int(l * proportion))
		leftDs = self.copy()
		leftDs.clear()
		rightDs = leftDs.copy()
		index = 0
		for seq in iter(self):
			if index in leftIndices:
				leftDs.newSequence()
				for sp in seq:
					leftDs.addSample(*sp)
			else:
				rightDs.newSequence()
				for sp in seq:
					rightDs.addSample(*sp)
			index += 1
		return leftDs, rightDs
